Class: Doom::Game::Combat::Projectile

Inherits:
Struct
  • Object
show all
Defined in:
lib/doom/game/combat.rb

Overview

owner is the Player who fired, so a kill can be credited. Monster projectiles leave it nil: DOOM gives no frag for a death by monster. damage_multiplier scales the damage a monster fireball deals on impact, so a skill's damage factor reaches the fireball it spawned. Player projectiles (rockets) carry 1.0.

Instance Attribute Summary collapse

Instance Attribute Details

#damage_multiplierObject

Returns the value of attribute damage_multiplier

Returns:

  • (Object)

    the current value of damage_multiplier



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def damage_multiplier
  @damage_multiplier
end

#dxObject

Returns the value of attribute dx

Returns:

  • (Object)

    the current value of dx



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def dx
  @dx
end

#dyObject

Returns the value of attribute dy

Returns:

  • (Object)

    the current value of dy



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def dy
  @dy
end

#dzObject

Returns the value of attribute dz

Returns:

  • (Object)

    the current value of dz



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def dz
  @dz
end

#ownerObject

Returns the value of attribute owner

Returns:

  • (Object)

    the current value of owner



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def owner
  @owner
end

#spawn_ticObject

Returns the value of attribute spawn_tic

Returns:

  • (Object)

    the current value of spawn_tic



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def spawn_tic
  @spawn_tic
end

#sprite_prefixObject

Returns the value of attribute sprite_prefix

Returns:

  • (Object)

    the current value of sprite_prefix



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def sprite_prefix
  @sprite_prefix
end

#targetObject

Returns the value of attribute target

Returns:

  • (Object)

    the current value of target



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def target
  @target
end

#typeObject

Returns the value of attribute type

Returns:

  • (Object)

    the current value of type



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def type
  @type
end

#xObject

Returns the value of attribute x

Returns:

  • (Object)

    the current value of x



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def x
  @x
end

#yObject

Returns the value of attribute y

Returns:

  • (Object)

    the current value of y



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def y
  @y
end

#zObject

Returns the value of attribute z

Returns:

  • (Object)

    the current value of z



103
104
105
# File 'lib/doom/game/combat.rb', line 103

def z
  @z
end