Class: Doom::Net::GameServer::Member
- Inherits:
-
Struct
- Object
- Struct
- Doom::Net::GameServer::Member
- Defined in:
- lib/doom/net/game_server.rb
Overview
A connected client's bookkeeping (named Member so it is not confused with Net::Client, the actual client program in the same module).
Instance Attribute Summary collapse
-
#host ⇒ Object
Returns the value of attribute host.
-
#joined ⇒ Object
Returns the value of attribute joined.
-
#last_seen ⇒ Object
Returns the value of attribute last_seen.
-
#player_id ⇒ Object
Returns the value of attribute player_id.
-
#port ⇒ Object
Returns the value of attribute port.
Instance Method Summary collapse
Instance Attribute Details
#host ⇒ Object
Returns the value of attribute host
33 34 35 |
# File 'lib/doom/net/game_server.rb', line 33 def host @host end |
#joined ⇒ Object
Returns the value of attribute joined
33 34 35 |
# File 'lib/doom/net/game_server.rb', line 33 def joined @joined end |
#last_seen ⇒ Object
Returns the value of attribute last_seen
33 34 35 |
# File 'lib/doom/net/game_server.rb', line 33 def last_seen @last_seen end |
#player_id ⇒ Object
Returns the value of attribute player_id
33 34 35 |
# File 'lib/doom/net/game_server.rb', line 33 def player_id @player_id end |
#port ⇒ Object
Returns the value of attribute port
33 34 35 |
# File 'lib/doom/net/game_server.rb', line 33 def port @port end |
Instance Method Details
#key ⇒ Object
34 |
# File 'lib/doom/net/game_server.rb', line 34 def key = "#{host}:#{port}" |